/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the mozCellML code.
 *
 * The Initial Developer of the Original Code is
 * The Bioengineering Institute, University of Auckland.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Andrew Miller <ak.miller@auckland.ac.nz> (original developer)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "mozCellMLSimpleGraphProvider.h"
#include "nsIRenderingContext.h"
#include <stdio.h>

NS_IMPL_ISUPPORTS1(mozCellMLSimpleGraphProvider, mozCellMLIGraphProvider)

mozCellMLSimpleGraphProvider::mozCellMLSimpleGraphProvider()
  : colours(nsnull), visible(nsnull), data(nsnull)
{
}

mozCellMLSimpleGraphProvider::~mozCellMLSimpleGraphProvider()
{
  if (data != nsnull)
    delete [] data;
  if (colours != nsnull)
    delete [] colours;
  if (visible != nsnull)
    delete [] visible;
  if (rangemax != nsnull)
    delete [] rangemax;
}

NS_IMETHODIMP
mozCellMLSimpleGraphProvider::Init(
                                   double iminBound,
                                   double imaxBound,
                                   PRUint32 invariables,
                                   PRUint32 *icolours,
                                   double igrid
                                  )
{
  minBound = iminBound;
  maxBound = imaxBound;
  grid = igrid;
  nvariables = invariables;

  colours = new PRUint32[nvariables];
  visible = new PRBool[nvariables];
  rangemax = new double[nvariables];
  memset(rangemax, 0, nvariables * sizeof(double));

  PRUint32 i;
  for (i = 0; i < nvariables; i++)
  {
    colours[i] = 0xFF000000 | (icolours[i] & 0xFFFFFF);
    visible[i] = PR_TRUE;
  }

  double npoints = ((maxBound - minBound) / grid);
  npointsTotal = ((npoints - (PRUint32)npoints) == 0) ?
                   ((PRUint32)npoints) : ((PRUint32)(npoints + 1));
  npointsNow = 0;

  data = new double[nvariables * npointsTotal];
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLSimpleGraphProvider::SetColour(PRUint32 traceId, PRUint32 newColour)
{
  if (traceId >= nvariables)
    return NS_ERROR_INVALID_ARG;
  colours[traceId] = 0xFF000000 | (newColour & 0xFFFFFF);
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLSimpleGraphProvider::SetVisibility(PRUint32 traceId,
                                            PRBool visibility)
{
  if (traceId >= nvariables)
    return NS_ERROR_INVALID_ARG;
  visible[traceId] = visibility;
  return NS_OK;
}

static inline double dabs(double v)
{
  if (v >= 0)
    return v;
  return -v;
}

NS_IMETHODIMP
mozCellMLSimpleGraphProvider::AddDataPoint(PRUint32 invariables,
                                           double* datapoint)
{
  if (invariables != nvariables)
    return NS_ERROR_INVALID_ARG;
  
  if (npointsNow >= npointsTotal)
    return NS_ERROR_FAILURE;

  PRUint32 i;
  PRUint32 offset = npointsNow * nvariables;

  for (i = 0; i < nvariables; i++)
  {
    data[i + offset] = datapoint[i];
    if (dabs(datapoint[i]) > rangemax[i] || npointsNow == 0)
      rangemax[i] = dabs(datapoint[i]);
  }
  npointsNow++;
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLSimpleGraphProvider::Paint(
                                    nsIRenderingContext* context,
                                    PRInt32 mX,
                                    PRInt32 mY,
                                    PRInt32 mWidth,
                                    PRInt32 mHeight
                                   )
{
  // Fill everything with white...
  context->SetColor(0xFFFFFF);
  context->FillRect(mX, mY, mWidth, mHeight);

  // We plot the entire domain across mX ... mX + mWidth...
  double xm, gridX;
  xm = ((double)mWidth) / (maxBound - minBound);
  gridX = xm * grid;

  context->SetColor(0x0);
  // We draw a X axis...
  PRUint32 centerY = mY + (mHeight / 2);
  context->DrawLine(mX, centerY, mX + mWidth, centerY);

  // If the range cuts bound=0, draw a Y axis too...
  if (maxBound >= 0 && minBound <= 0)
  {
    PRUint32 originX = (PRUint32)(-minBound * xm) + mX;
    context->DrawLine(originX, mY, originX, mY + mHeight);
  }

  // Draw the data on...
  PRUint32 i;
  for (i = 0; i < nvariables; i++)
  {
    if (!visible[i])
      continue;
    context->SetColor(colours[i]);
    PRUint32 offset, stopOffset = nvariables * npointsNow;
    double coordX = (double)mX;
    double rm = rangemax[i];
    if (rm == 0)
      rm = 1.0;
    double ym = ((double)mHeight) / (rm * 2.0);
    PRUint32 lastX = 0, lastY = 0;
    for (offset = 0;
         offset < stopOffset;
         offset += nvariables, coordX += gridX)
    {
      // Carry out the range transformation...
      PRUint32 coordY = (PRUint32)(mHeight / 2 - ym * data[offset + i]) + mY;
      if (offset != 0)
        context->DrawLine(lastX, lastY, (PRUint32)coordX, coordY);
      lastX = (PRUint32)coordX;
      lastY = coordY;
    }
  }
  return NS_OK;
}
